 /*********************************************************************/
 /*																	  */
 /*    MQTM - Intelligent MQSeries Trigger Monitor                    */
 /*																	  */
 /*    (C) Alphacourt Limited 2000                                    */
 /*																	  */
 /*     Usage:														  */
 /*            MQTM <init q name> <Q Mgr name>                        */
 /*																	  */
 /*     Note:  The queue manager name can be omitted and the default  */
 /*            queue manager will be used.  The initiation queue	  */
 /*            must be specied.										  */
 /*																	  */
 /*     For further details see README.txt file                       */
 /*                                                                   */
 /*********************************************************************/

 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <cmqc.h>
 #include <signal.h>

 /**************************************************** 
 ** CONSTANTS
 *****************************************************/
 #define LETTER_M								109
 #define LETTER_Q								113
 #define LETTER_T								116
 #define LETTER_I								105
 #define LETTER_D								100
 #define MINUS_SIGN								45
 #define MAX_OF_999								4
 #define MAX_OF_99								3
 #define MAX_FILE_NAME_LENGTH					128
 /**************************************************** 
 ** EXIT CODES
 *****************************************************/
 #define MQ_Q_NAME_TOO_LONG						21
 #define NO_INIT_QUEUE_NAME_SPECIFIED			22
 #define MQ_Q_MGR_NAME_TOO_LONG					23
 #define MAX_TRIGGER_DEPTH_EXCEEDED				24
 #define MAX_INSTANCES_EXCEEDED					25
 #define NO_OPTION_SPECIFIED_AFTER_MINUS_SIGN	26
 #define NO_LOGFILE_NAME_SPECIFIED				27
 #define ERROR_OPENING_LOGFILE					28
 #define MAX_FILE_NAME_LENGTH_EXCEEDED			29

 /**************************************************** 
 ** STRING DEFINES
 *****************************************************/
 
 /**************************************************** 
 ** FUNCTION PROTOTYPES
 *****************************************************/

 int getQAttrs (MQHCONN Hcon, MQCHAR48 TQName,
				MQLONG *depth, MQLONG *input,
				FILE *LFile, int *Debug);
 
 void usage_msg(void);

 void handle_signal( int signum);

 /****************************************************
 ** GLOBAL VARIABLES
 *****************************************************/

 int finished;

 /**************************************************** 
 ** MAIN PART OF APPLICATION
 *****************************************************/


 int main(int argc, char **argv)
 {
   MQOD     od = {MQOD_DEFAULT};    
   MQMD     md = {MQMD_DEFAULT};    
   MQGMO    gmo = {MQGMO_DEFAULT};   
   MQTM     trig;                   

   MQHCONN  Hcon;                   
   MQHOBJ   Hobj;                   
   MQLONG   O_options;              
   MQLONG   CompCode;               
   MQLONG   Reason;                 
   MQLONG   buflen;                 
   MQLONG   triglen;                

   MQCHAR   command[1200];          
   MQCHAR   ApplID[MQ_PROCESS_APPL_ID_LENGTH];                
   MQCHAR   TrigData[685]; /* Sizeof a Trig Structure = 684 Also need 1 for teminating NULL */                
   MQCHAR   other_parms[128];                
   MQCHAR   EnvData[MQ_PROCESS_ENV_DATA_LENGTH] = "";
   MQCHAR   temp_env[MQ_PROCESS_ENV_DATA_LENGTH] = "";
   MQLONG	depth = 0;
   MQLONG	input = 0;
   MQCHAR48 TQName;

   /**************************************************** 
   ** INPUT PARAMS TO PROGRAM
   *****************************************************/
   MQCHAR48	IQName;
   MQCHAR48	QMName;
   char		LogFileName[MAX_FILE_NAME_LENGTH];
   
   /**************************************************** 
   ** PROGRAM INTEGERS
   *****************************************************/
   int		i;	
   int		curOpt,nextOpt; /*used for parsing the input params */
   int		initQSet = 0;
   int		Debug = 0 ;
   int		tDepth = 0;
   int		maxI = 0;
   int		temp_tDepth = 0;
   int		temp_maxI = 0;

   /**************************************************** 
   ** PROGRAM POINTERS
   *****************************************************/
   char     *field;
   char		*pArgv;
   FILE		*LFile;

   /****************************************************
   ** Set up signal handler
   *****************************************************/
    /* UNIX specific are commented out */
	/*signal(SIGHUP, LFile,  handle_signal);
    signal(SIGQUIT, handle_signal);
    signal(SIGTRAP, handle_signal);
    signal(SIGEMT,  handle_signal);
    signal(SIGBUS,  handle_signal);
    signal(SIGSYS,  handle_signal);
    signal(SIGPIPE, handle_signal);*/
    signal(SIGTERM, handle_signal);
    signal(SIGINT,  handle_signal);
    signal(SIGILL,  handle_signal);
    signal(SIGABRT, handle_signal);
    signal(SIGFPE,  handle_signal);
    signal(SIGSEGV, handle_signal);


   printf("**************************************************************\n");
   printf("    Intelligent Trigger Monitor. (C) Alphacourt 2001 Ltd\n");
   printf("**************************************************************\n");

   /*************************************************************************
   ** This is going to be the check to make sure all required params
   ** have been passed in on the command line
   **************************************************************************/
   if (argc < 2)
   {
     printf("Required parameter missing - initiation queue name\n");
     exit(99);     
   }

   /*************************************************************************
   ** Not interested in value argv[0] because this is the executable name 
   **************************************************************************/
   QMName[0] = 0;        /* get ready to use the default queue manager */
   
   LogFileName[0] = 0;

   finished = 0;

   for (i=1;i<argc;i++)
   {
	   pArgv=argv[i];
	   /* If the value of *pArgv is MINUS_SIGN then we know that there
	   ** is a parameter option to follow. We should get this param option
       ** and depending on its value set the required parameters for the rest 
       ** of the execution of this program Possible params are -q -m -t -i -d.
	   ** These will be 
	   ** -q : Initiation Queue Name.
	   ** -m : Qmanager name.
	   ** -i : Max Instances of this program <Optional>
	   ** -t : Max queue depth before another instance will be invoked <Optional>
	   ** -d : debug option that enables dignostic information to be written to file <Optional>
       */
	   curOpt = (int)*pArgv; /* Value of first element of argv array       */
	   switch(curOpt)
	   {
	   case MINUS_SIGN : //printf("Got one %s\n",argv[i]);
		   nextOpt = *++pArgv;
   	       switch(nextOpt)
		   {
		   case LETTER_Q : 
			   pArgv=argv[i+1]; /*Point to the next value in the argv array */
			   if ((pArgv == NULL) || ((int)*pArgv==MINUS_SIGN))
			   {
			       printf("\nNo Initiation Queue Name Specified\n");
				   usage_msg();
			       exit(NO_INIT_QUEUE_NAME_SPECIFIED);
			   }
			   else
			   {
				   if (strlen(pArgv)>(MQ_Q_NAME_LENGTH-1))
				   {
				       printf("\nQueue Name Too Long\n"); 
				       exit(MQ_Q_NAME_TOO_LONG);
				   }
				   strcpy(IQName,pArgv);
			       initQSet=1;
			   }
			   break;
		   case LETTER_M :
			   pArgv=argv[i+1]; /*Point to the next value in the argv array */
			   if ((pArgv == NULL) || ((int)*pArgv==MINUS_SIGN)) 
			   {
				   printf("\nNo Queue Manager Name Specified\n");
				   printf("Preparing To Use Default....\n");
			   }
			   else
			   {
				   if (strlen(pArgv)>(MQ_Q_MGR_NAME_LENGTH-1))
				   {
					   printf("\nQueue Manager Name Too Long\n");
					   usage_msg();
					   exit(MQ_Q_MGR_NAME_TOO_LONG);
				   }
				   strcpy(QMName,pArgv);
			   }
			   break;
		   case LETTER_T : 
			   pArgv=argv[i+1]; /*Point to the next value in the argv array */
			   if ((pArgv == NULL) || ((int)*pArgv==MINUS_SIGN)) 
			   {
				   printf("\nNo Trigger Queue Depth Specified\n");
			   }
			   else
			   {
				   if (strlen(pArgv)>(MAX_OF_999-1))
				   {
					   printf("\nMax Trigger Depth For Application Exceeded - Must Be < 999\n"); 
					   usage_msg();
					   exit(MAX_TRIGGER_DEPTH_EXCEEDED);
				   }
				   tDepth=atoi(pArgv);
			   }
			   break;
		   case LETTER_I : 
			   pArgv=argv[i+1]; /*Point to the next value in the argv array */
			   if ((pArgv == NULL) || ((int)*pArgv==MINUS_SIGN)) 
			   {
				   printf("\nNo Max Instances Value Specified\n");
			   }
			   else
			   {
				   if (strlen(pArgv)>(MAX_OF_99-1))
				   {
					   printf("\nMax Instances For Application Exceeded - Must Be < 99\n"); 
					   usage_msg();
				       exit(MAX_INSTANCES_EXCEEDED);
				   }
				   maxI=atoi(pArgv);
			   }
			   break;
		   case LETTER_D :
			   pArgv=argv[i+1]; /*Point to the next value in the argv array */
			   if ((pArgv == NULL) || ((int)*pArgv==MINUS_SIGN)) 
			   {
				   printf("\nNo Log File Name Specified\n");
				   usage_msg();
				   exit(NO_LOGFILE_NAME_SPECIFIED);
			   }
			   else
			   {
				   if (strlen(pArgv)>(MAX_FILE_NAME_LENGTH-1))
				   {
					   printf("\nMax Length For Filename Exceeded - Must Be < 128 Chars\n"); 
					   usage_msg();
					   exit(MAX_FILE_NAME_LENGTH_EXCEEDED);
				   }
				   strcpy(LogFileName,pArgv);
				   Debug=1;
			   }
			   break;
		   default : printf("No Such Option Available\n");
			   usage_msg();
			   exit(NO_OPTION_SPECIFIED_AFTER_MINUS_SIGN);
			   break;
		   }
		   break;
	   default : break;
	   }
   }

   /*************************************************************************
   ** We have parsed the input Params so we must make sure that the 
   ** initiation queue was specified.  
   **************************************************************************/
   if (!initQSet) 
   {
	   printf("\nNo Initiation Queue Name Specified\n");
	   usage_msg();
	   exit(NO_INIT_QUEUE_NAME_SPECIFIED);
   }

   /*************************************************************************
   ** If Specified, open the Debug logfile  
   **************************************************************************/
   if (Debug)
   {
	   if ((LFile=fopen(LogFileName,"a+"))==NULL)
	   {
		   printf("Error Opening File %s\nExiting",LogFileName);
		   exit(ERROR_OPENING_LOGFILE);
	   }
	}

   /*************************************************************************
   ** Print More Verbose Messages to the Logfile if specified  
   **************************************************************************/
   if (Debug)
   {
	   fprintf(LFile,"MAIN_PG: **************************************************************\n");
	   fprintf(LFile,"MAIN_PG: \tIntelligent Trigger Monitor. (C) Alphacourt 2001 Ltd\n");
	   fprintf(LFile,"MAIN_PG: **************************************************************\n");
	   fprintf(LFile,"MAIN_PG: You are running the program with the following parameters :-\n");
	   fprintf(LFile,"MAIN_PG: \nMAIN_PG: Queue Manager Name      : %s \n",QMName);
	   fprintf(LFile,"MAIN_PG: \nMAIN_PG: Initiation Queue Name   : %s \n",IQName);
	   fprintf(LFile,"MAIN_PG: \nMAIN_PG: Queue Depth Before Next : %d\n",tDepth);
	   fprintf(LFile,"MAIN_PG: Application Start.\n");
	   fprintf(LFile,"MAIN_PG: \nMAIN_PG: Maximum Number Of       : %d \n",maxI);
	   fprintf(LFile,"MAIN_PG: Application Instances.\n");
	   fprintf(LFile,"MAIN_PG: \nMAIN_PG: Debug Logfile Name      : %s \n",LogFileName);
	   fprintf(LFile,"MAIN_PG: \nMAIN_PG: **************************************************************\n");
   }

   strcpy(od.ObjectName, IQName);
   
   /*************************************************************************
   ** Connect to the queue manager and open the init queue  
   **************************************************************************/

   if (Debug)
   {
	   if (!strlen(QMName)) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Connecting To The Default Queue Manager ......\n");
	   else
		   fprintf(LFile,"MAIN_PG:\nMAIN_PG: Connecting To Queue Manager %s ......\n",QMName);
   }

   MQCONN(QMName, &Hcon, &CompCode, &Reason);               

   if (CompCode == MQCC_FAILED)
   {
     if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Cannot connect to queue manager: Reason code %ld\n", Reason);
	 printf("\nCannot connect to queue manager: Reason code %ld\n", Reason);
	 if (Debug) fclose(LFile);
     exit(Reason);
   }
   
   if (Debug)
   {
	   if (!strlen(QMName)) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Connected To Default Queue Manager.\n");
		   //fflush(LFile);
	   else
		   fprintf(LFile,"MAIN_PG:\nMAIN_PG: Connected to Queue Manager %s.\n",QMName);
   }

   /*************************************************************************
   ** Set Up the Open Options for the Initiation Queue  
   **************************************************************************/

   O_options = MQOO_INPUT_AS_Q_DEF 
             + MQOO_FAIL_IF_QUIESCING; 

   /*************************************************************************
   ** Try to Open the Initiation Queue  
   **************************************************************************/

   if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Opening the Initiation Queue %s ......\n",IQName);
   
   MQOPEN(Hcon, &od, O_options, &Hobj, &CompCode, &Reason);                

   if (Reason != MQRC_NONE)
   {
	   if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Cannot Open Initiation Queue. Reason %ld\n", Reason);
	   printf("Cannot open Initiation queue.  Reason %ld\n", Reason);
	   if (Debug) fclose(LFile);
	   exit(Reason);
   }

   if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Initiation Queue Opened Successfully.\n");
 
   /*************************************************************************
   ** Set up the parameters for the GET request       
   **************************************************************************/
   
   buflen = sizeof(trig);             /* size of all trigger messages   */
   gmo.Version = MQGMO_VERSION_2;     /* Avoid need to reset Message */
   gmo.MatchOptions = MQMO_NONE;      /* ID and Correlation ID after */
                                      /* every MQGET                 */
   gmo.Options = MQGMO_WAIT             
     + MQGMO_FAIL_IF_QUIESCING          
     + MQGMO_ACCEPT_TRUNCATED_MSG;      
   gmo.WaitInterval = MQWI_UNLIMITED;   

   /*************************************************************************
   **   MAIN PROCESSING LOOP - loop forever, getting messages
   **						   and processing them.
   **************************************************************************/
   
   while ((CompCode != MQCC_FAILED) && (!finished))
   {
	 if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Waiting for trigger message.....\n");
	 printf("Waiting for trigger message.....\n");
	 MQGET(Hcon, Hobj, &md, &gmo, buflen, &trig, &triglen, &CompCode, &Reason);            

	 /*
	 ** Should be able to remove these lines when the program is finished 
	 */
	 //fflush(LFile);

     if (Reason != MQRC_NONE)     /* Deal with an MQSeries error  */
     {
       if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Error received on MQGET request.  Reason: %ld\n", Reason);
	   printf("Error received on MQGET request.  Reason: %ld\n", Reason);
     }

	 /*
	 ** Should be able to remove these lines when the program is finished 
	 */
	 //fflush(LFile);

     if (CompCode != MQCC_FAILED) /* Everything ok, process the message */
     {
		 if (triglen != buflen)     /* Message too big ?                  */
		 {
			 if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: DataLength = %ld?\n", triglen);
			 printf("DataLength = %ld?\n", triglen);
		 }
		 else
		 {
			 if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Trigger received.....\n");
			 printf("Trigger received.....\n");

			 /*
			 ** Should be able to remove these lines when the program is finished 
			 */
			 //fflush(LFile);


			 /*************************************************************************
			 **   Assign and Null Terminate Trigger Paramaters
			 **************************************************************************/
			 memcpy(ApplID, trig.ApplId, sizeof(trig.ApplId));

			 /* NULL Terminate the ApplId */
			 for (i=sizeof(trig.ApplId)-1; i>=0; i--)
				 if (ApplID[i] != ' ')
					 break;
				 ApplID[i+1] = '\0';

			 memcpy(EnvData, trig.EnvData, sizeof(trig.EnvData));

			 /* NULL Terminate the EnvData */
			 for (i=sizeof(trig.EnvData)-1; i>=0; i--)
				 if (EnvData[i] != ' ')
					 break;
				 EnvData[i+1] = '\0';

			 memcpy(TQName, trig.QName, sizeof(trig.QName));
			 
			 /* NULL Terminate the TQName */
			 for (i=sizeof(trig.QName)-1; i>=0; i--)
				 if (TQName[i] != ' ')
					 break;
				 TQName[i+1] = '\0';
				 
			 if (Debug)
			 {
				 fprintf(LFile,"MAIN_PG:\nMAIN_PG: Received ApplId  -> %s\n",ApplID);
				 fprintf(LFile,"MAIN_PG: Received EnvData -> %s\n",EnvData);
				 fprintf(LFile,"MAIN_PG: Received TQName  -> %s\n",TQName);
			 }
			  	     
			 /*
			 ** Should be able to remove these lines when the program is finished 
			 */
			 //fflush(LFile);

			 /*************************************************************************
			 **   Reset the trigger structure Version and ApplType
			 **************************************************************************/

			 memcpy(&trig.Version,  "   1", 4); /* replace integers */
			 memcpy(&trig.ApplType, "    ", 4);

			 /*************************************************************************
			 **   Copy the contents of the memory at address trig for a length of triglen
			 **************************************************************************/

			 memcpy(TrigData, &trig, triglen);        /* copy modified trigger */
			 TrigData[triglen+1] = '\0';
			 
			 /*
			 ** Should be able to remove these lines when the program is finished 
			 */
			 //fflush(LFile);
			 
			 /*************************************************************************
			 ** Parse the envdata looking for the magic characters
			 ** -i controls the maximum number of instances       
			 ** -t controls the trigger depth
			 ** Note : If these values are present then they will over-ride the 
			 ** values given at the command line when the trigger monitor 
			 ** was initialised
			 **************************************************************************/

			 /*************************************************************************
			 ** We need to create a temporary holding for the current maxI and tDepth
			 ** values because we want to restore the command line settings if the 
			 ** EnvData is used to over-ride them. This is because two applications 
			 ** might be using the same TM and require different settings for maxI and 
			 ** tDepth
			 **************************************************************************/

			 temp_tDepth = tDepth;
			 temp_maxI = maxI;

			 /*************************************************************************
			 **   Create a temporary EnvData string since strtok replaces chosen tokens
			 ** with Nulls therefore affecting the value of the original string if it 
			 ** was to be used later on within the program.
			 **************************************************************************/

			 memset(temp_env,'\0',sizeof(temp_env));

			 strcpy(temp_env,EnvData);

			 field = strtok(temp_env, " ");
			 strcpy(other_parms,"");
			 while ( field != 0 )
			 {
				 if ( strcmp(field,"-i") == 0 )
				 {
					 field = strtok(NULL, " ");
					 if (maxI != atoi(field))
					 {
						 if (Debug)
						 {
							 fprintf(LFile,"MAIN_PG:\nMAIN_PG: Replacing Initial Value Of -i field.\n");
							 fprintf(LFile,"MAIN_PG: Previous Value Was   : %d\n",maxI);
							 fprintf(LFile,"MAIN_PG: New Value To Be Used : %d\n",atoi(field));
							 //fflush(LFile);
						 }
						 maxI = atoi(field);
					 }
				 }
				 else
				 {
					 if (strcmp(field, "-t") ==0 )
					 {
						 field = strtok(NULL, " ");
						 if (tDepth != atoi(field))
						 {
							 if (Debug)
							 {
								 fprintf(LFile,"MAIN_PG:\nMAIN_PG: Replacing Initial Value Of -t field.\n");
								 fprintf(LFile,"MAIN_PG: Previous Value Was   : %d\n",tDepth);
								 fprintf(LFile,"MAIN_PG: New Value To Be Used : %d\n",atoi(field));
								 //fflush(LFile);
							 }
	 						 tDepth= atoi(field);
						 }
					 }
					 else
					 {
						 /* Build a string of other params if found */
						 strcat(other_parms, field);
						 strcat(other_parms, " ");
					 }
				 }
				 field = strtok(NULL, " "); 
			 }
			 
			 
			 /*************************************************************************
			 ** Having understood the tigger message, go look at the application
			 ** queue and figure out what state it is in.
			 **************************************************************************/
			 
			 CompCode = getQAttrs(Hcon, TQName, &depth, &input, &*LFile, &Debug);
			 if (CompCode != 0)
			 {
				 if (Debug) fprintf(LFile,"MAIN_PG:\nMAIN_PG: Unable to examine application queue %s, terminating.\n",TQName);
				 printf("Unable to examine application queue, terminating.");
			 }
			 else
			 {
				 /* Now decide whether to trigger or not base on the rules */
				 if ( (tDepth == 0) || (input <= depth/tDepth) )  /* Do we need another ? */
				 {
					 if ( (maxI == 0) || (input < maxI) )    /* Are we at max Instances ? */
					 {
						 sprintf(command, "%s \"%s%s\" %s", ApplID, TrigData , QMName , other_parms);
						 
						 /*   Finally, call the program ! */
						 if (Debug) 
						 {
							 fprintf(LFile,"MAIN_PG:\nMAIN_PG: Issuing Command : %s;\n", command);
							 //fflush(LFile);
						 }
						 printf("About to Issue : %s\n",command);
						 system(command);
					 }
					 else
					 {
						 if (Debug)
						 {
							 fprintf(LFile,"MAIN_PG:\nMAIN_PG: Trigger received, but already at maximum Processes\n");
							 fprintf(LFile,"MAIN_PG: Current Settings Are :\n");
							 fprintf(LFile,"MAIN_PG: Trigger Queue Depth (-t)          : %d\n",tDepth);
							 fprintf(LFile,"MAIN_PG: Max Instances of Application (-i) : %d\n",maxI);
							 fprintf(LFile,"MAIN_PG: You may change these values in the process definition dynamically.\n");
							 //fflush(LFile);
						 }
						 printf("Trigger received, but already at maximum Processes\n");
					 }
				 }
			 }
			 /*************************************************************************
			 ** Reset the values for maxI and tDepth
			 **************************************************************************/
			 maxI = temp_maxI;
			 tDepth = temp_tDepth;
		 }   
      }     
   }       

   /*   Close the initiation queue and disconnect from queue manager */

   MQCLOSE(Hcon, &Hobj, 0, &CompCode, &Reason);             
   
   MQDISC(&Hcon, &CompCode, &Reason);                

   if (Debug) fclose(LFile);

   return(0);
 }

 /******************************************************************
 **   getQAttrs routine - used to figure out the current depth     
 **                       and the number of processes that already 
 **						  have the queue open for input.           
 **						  Uses the MQSeries inquire API            
 *******************************************************************/
   
 int getQAttrs(MQHCONN Hcon, MQCHAR48 TQName, MQLONG *depth, MQLONG *input, FILE *LFile, int *Debug)
 {
	MQOD     odI = {MQOD_DEFAULT};    
	MQLONG 	 O_options;
	MQHOBJ	 Hinq;
	MQLONG	 CompCode;
	MQLONG	 Reason;
	MQLONG   Select[2];
	MQLONG   IAV[2];

    strncpy(odI.ObjectName, TQName, MQ_Q_NAME_LENGTH);

	/* Open the application for Inquiry only */

    O_options = MQOO_INQUIRE;    
    MQOPEN(Hcon, &odI, O_options, &Hinq, &CompCode, &Reason);            

	if (Reason != MQRC_NONE)
	{
		if (Debug) fprintf(LFile,"GET_ATT:\nGET_ATT: Error opening application queue, Reason %ld\n", Reason);
		printf("Error opening application queue, Reason %ld\n", Reason);
		return CompCode;
	}

    /* Set up the two attributes we are interested in and issue the */
	/* inquire call.												*/

	Select[0] = MQIA_CURRENT_Q_DEPTH;
	Select[1] = MQIA_OPEN_INPUT_COUNT;       

    MQINQ(Hcon, Hinq, 2L, Select, 2L, IAV, 0L, NULL, &CompCode, &Reason);        

	if (Reason != MQRC_NONE)
	{
		if (Debug) fprintf(LFile,"GET_ATT:\nGET_ATT: Error inquiring about application queue, Reason %ld\n", Reason);
		printf("Error inquiring about application queue, Reason %ld\n", Reason);
		return CompCode;
	}
	         
    /* Close the queue again and return the two values              */

    MQCLOSE(Hcon, &Hinq, 0L, &CompCode, &Reason);          
	
	*depth = IAV[0];
	*input = IAV[1];
	return MQCC_OK;
 }

 /******************************************************************
 **   usage_msg -	used to print to stdout the usage message for 
 **					the application. This function is called if any
 **					invalid combination of input parameters is 
 **					specified or there is no INITQ given.  
 *******************************************************************/
 
 void usage_msg(void)

 {
	 printf("\nMAIN_PG: Usage : mqtm -q <QN> -m <QMN> -t <TD> -i <MI> -d <FN>\n");
	 printf("MAIN_PG: Where :\n");
	 printf("MAIN_PG: \tQN  => Initiation Queue Name.               (Required)\n");
	 printf("MAIN_PG: \tQMN => Queue Manager Name.                  (Optional)\n");
	 printf("MAIN_PG: \tTD  => Trigger Depth.                       (Optional)\n");
	 printf("MAIN_PG: \tMI  => Maximum number of program instances. (Optional)\n");
	 printf("MAIN_PG: \tFN  => Debug File Name.                     (Optional)\n");
 }


 /******************************************************************
 **   handle_signal -	used to control any signal events such that
 **                     the application will shut down gracefully
 *******************************************************************/

 void handle_signal( int signum)
 {

    finished = 1;

    return;

 }

